﻿using Infragistics.Windows.OutlookBar;
using System;
using System.Diagnostics;
using System.Windows;
using System.Windows.Input;

namespace XamOutlookBarKeyboardShortcuts
{
    public class InputBinding : DependencyObject
    {
        public static readonly DependencyProperty InputBindingBehaviorProperty =
            DependencyProperty.RegisterAttached("InputBindingBehavior", typeof(XamOutlookBarKeyBindingBehavior), typeof(InputBinding), new PropertyMetadata(null));

        public static readonly DependencyProperty EnableInputBindingsProperty =
            DependencyProperty.RegisterAttached("EnableInputBindings", typeof(bool), typeof(InputBinding), new PropertyMetadata(false, new PropertyChangedCallback(EnableInputBindingsChanged)));
        public static bool GetEnableInputBindings(DependencyObject obj)
        {
            return (bool)obj.GetValue(EnableInputBindingsProperty);
        }
        public static void SetEnableInputBindings(DependencyObject obj, bool value)
        {
            obj.SetValue(EnableInputBindingsProperty, value);
        }

        private static void EnableInputBindingsChanged(DependencyObject d, DependencyPropertyChangedEventArgs e)
        {
            XamOutlookBar outlookBar = d as XamOutlookBar;
            bool isEnabled = (bool)e.NewValue;

            if (outlookBar != null)
            {
                XamOutlookBarKeyBindingBehavior behavior = GetOrCreateBehavior(outlookBar);

                if (isEnabled)
                    behavior.Attach();
                else
                    behavior.Detach();
            }
        }

        private static XamOutlookBarKeyBindingBehavior GetOrCreateBehavior(XamOutlookBar outlookBar)
        {
            XamOutlookBarKeyBindingBehavior behavior = outlookBar.GetValue(InputBindingBehaviorProperty) as XamOutlookBarKeyBindingBehavior;
            if (behavior == null)
            {
                behavior = new XamOutlookBarKeyBindingBehavior(outlookBar);
                outlookBar.SetValue(InputBindingBehaviorProperty, behavior);
            }

            return behavior;
        }
    }

    public class XamOutlookBarKeyBindingBehavior : InputBindingBehaviorBase<XamOutlookBar>
    {
        Window _parentWindow;

        public XamOutlookBarKeyBindingBehavior(XamOutlookBar outlookBar)
            : base(outlookBar)
        {

        }

        public override void Attach()
        {
            //since we want to listen for all key events no matter which control has focus, we need to listen at the Window level
            //otherwise the KeyUp event will never execute
            if (_parentWindow == null)
                _parentWindow = Window.GetWindow(TargetObject);

            if (_parentWindow != null)
                _parentWindow.AddHandler(Keyboard.KeyUpEvent, (KeyEventHandler)HandleKeyUp, true);
        }

        public override void Detach()
        {
            if (_parentWindow != null)
                _parentWindow.RemoveHandler(Keyboard.KeyUpEvent, (KeyEventHandler)HandleKeyUp);
        }

        void HandleKeyUp(object sender, System.Windows.Input.KeyEventArgs e)
        {
            try
            {
                //We only want to check for shorcuts if we are dealing with modifier keys.
                if (Keyboard.Modifiers == ModifierKeys.None)
                    return;

                foreach (OutlookBarGroup group in TargetObject.Groups)
                {
                    foreach (KeyBinding binding in group.InputBindings)
                    {
                        if (binding.Modifiers == Keyboard.Modifiers && binding.Key == e.Key)
                        {
                            group.IsSelected = true;
                            return;
                        }
                    }
                }
            }
            catch (Exception ex)
            {
                Debug.WriteLine(ex.Message);
            }
        }
    }

    public abstract class InputBindingBehaviorBase<T> where T : UIElement
    {
        private readonly WeakReference _targetObject;
        protected T TargetObject
        {
            get { return _targetObject.Target as T; }
        }

        public InputBindingBehaviorBase(T targetObject)
        {
            _targetObject = new WeakReference(targetObject);
        }

        public abstract void Attach();
        public abstract void Detach();

    }
}
